<?php declare(strict_types=1);

namespace Addons\Integrations\Pusher\Listeners;

use Illuminate\Support\Facades\Config;
use Illuminate\View\View;
use SupportPal\Support\Json\Json;

use function array_merge;
use function e;
use function trans;
use function url;

class AddMetaConfig
{
    /**
     * @return mixed
     */
    public function hook(?View $view = null)
    {
        $data = $this->data($view)->encode();
        if ($data !== false) {
            $data = e($data);
        }

        return <<<EOT
<meta name="pusher_config" content="$data" />
EOT;
    }

    private function data(?View $view = null): Json
    {
        return new Json([
            'key'    => Config::get('broadcasting.connections.pusher.key'),
            'params' => array_merge($this->defaultParams($view), $this->serverParams()),
            'lang'   => [
                'unable_to_receive_updates' => trans('integrations.pusher_unable_to_receive_updates'),
            ],
        ]);
    }

    /**
     * @return mixed[]
     */
    protected function defaultParams(?View $view = null): array
    {
        return ['authEndpoint' => url('broadcasting/auth')];
    }

    /**
     * @return mixed[]
     */
    protected function serverParams(): array
    {
        $params = [];
        $server = Config::get('broadcasting.connections.pusher.options');

        // set hosting type.
        $params['hosting'] = $server['hosting'] ?? '0';

        // https://beyondco.de/docs/laravel-websockets/basic-usage/pusher#usage-with-laravel-echo
        $params['wsHost'] = $server['external_host'] ?? null;
        $params['wsPort'] = $server['external_port'] ?? null;
        $params['forceTLS'] = $server['encryption'] ?? false;

        // pusher.com
        $params['cluster'] = $server['cluster'] ?? null;

        return $params;
    }
}
