<?php declare(strict_types=1);

namespace Addons\Reports\CannedResponses\Reports\Cards\Usage;

use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Report\Addon\Dashboards\Cards\TableCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Ticket\Models\CannedResponseUsage;
use Illuminate\Support\Facades\DB;

use function e;
use function route;
use function sprintf;
use function trans;

/**
 * @phpstan-extends TableCard<CannedResponseUsage>
 */
class CannedResponseUsageTable extends TableCard
{
    public function name(): string
    {
        return trans('report.most_used_responses');
    }

    /**
     * @return QueryBuilder<CannedResponseUsage>
     */
    public function query(): QueryBuilder
    {
        return QueryBuilder::for(CannedResponseUsage::class)
            ->join('canned_response', 'operator_canned_response_usage.response_id', '=', 'canned_response.id')
            ->groupBy('operator_canned_response_usage.response_id')
            ->select('canned_response.id', 'canned_response.name', DB::raw('sum(times) as sum'));
    }

    public function columns(): array
    {
        return [
            $this->nameColumn(),
            $this->sumColumn(),
        ];
    }

    /**
     * @return mixed[]
     */
    public function tableOptions(): array
    {
        return [
            'ordering' => true,
            'order'    => [[1, 'desc']]
        ];
    }

    protected function nameColumn(): Column
    {
        return (new Column(trans('general.name')))
            ->setId('name')
            ->setCsvValue(fn($model) => $model->name)
            ->setValue(function ($model) {
                $route = route('ticket.operator.cannedresponse.edit', $model->id);

                return sprintf('<a href="%s" target="_blank">%s</a>', e($route), e($model->name));
            }, false);
    }

    protected function sumColumn(): Column
    {
        return (new Column(trans('general.count')))
            ->setId('sum')
            ->setValue(fn($model) => $model->sum);
    }
}
