<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Controllers;

use App\Modules\Core\Models\FeedbackFormFieldValue;
use App\Modules\Core\Models\FeedbackLog;
use App\Modules\Selfservice\Models\Article;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;

use function sprintf;

/**
 * @template TModel of Ticket|Article
 * @template TFeedbackLog of FeedbackLog
 * @template TFeedbackFormFieldValue of FeedbackFormFieldValue
 */
trait ModelConcerns
{
    /**
     * @return TModel
     */
    abstract public function model();

    protected function getQualifiedName(string $table, string $column): string
    {
        return sprintf('%s.%s', $table, $column);
    }

    /**
     * @return HasOne<TFeedbackLog>
     */
    protected function logRelation(): HasOne
    {
        /** @var HasOne<TFeedbackLog> $relation */
        $relation = $this->model()->feedback();

        return $relation;
    }

    /**
     * @return TFeedbackLog
     */
    protected function logModel()
    {
        /** @var TFeedbackLog $model */
        $model = $this->logRelation()->getRelated();

        return $model;
    }

    /**
     * @return HasMany<TFeedbackFormFieldValue>
     */
    protected function valueRelation(): HasMany
    {
        /** @var HasMany<TFeedbackFormFieldValue> $relation */
        $relation = $this->logModel()->values();

        return $relation;
    }

    /**
     * @return TFeedbackFormFieldValue
     */
    protected function valueModel()
    {
        /** @var TFeedbackFormFieldValue $model */
        $model = $this->valueRelation()->getRelated();

        return $model;
    }
}
