<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports\Articles;

use Addons\Reports\Feedback\Reports\Cards\FeedbackLogTable as BaseFeedbackLogTable;
use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Selfservice\Models\Article;
use App\Modules\Selfservice\Models\FeedbackFormFieldValue;
use App\Modules\Selfservice\Models\FeedbackLog;

use function e;
use function route;
use function sprintf;
use function trans;
use function trans_choice;

/**
 * @extends BaseFeedbackLogTable<Article, FeedbackLog, FeedbackFormFieldValue>
 */
class FeedbackLogTable extends BaseFeedbackLogTable
{
    /** @var string[] */
    protected array $with = ['feedback.user'];

    public function columns(): array
    {
        return [
            $this->dateColumn(),
            $this->articleColumn(),
            $this->userColumn(),
            $this->ratingColumn(),
            $this->actionsColumn('selfservice.operator.feedback.show'),
        ];
    }

    public function model()
    {
        return new Article;
    }

    protected function articleColumn(): Column
    {
        return (new Column(trans_choice('selfservice.article', 1)))
            ->setCsvValue(fn($model) => $model->title)
            ->setValue(function ($model) {
                $route = route('selfservice.operator.article.edit', [$model->id]);

                return sprintf('<a href="%s">%s</a>', e($route), e($model->title));
            }, false);
    }

    protected function userColumn(): Column
    {
        return (new Column(trans('report.feedback_user_name_or_ip')))
            ->setCsvValue(fn($model) => $model->feedback->user?->formatted_name ?? $model->feedback->user_ip)
            ->setValue(function ($model) {
                if (! $model->feedback->user_id) {
                    return e($model->feedback->user_ip);
                }

                $route = route('user.operator.user.edit', $model->feedback->user_id);

                return sprintf('<a href="%s" target="_blank">%s</a>', e($route), e($model->feedback->user?->formatted_name ?? ''));
            }, false);
    }
}
