<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports\Cards;

use Addons\Reports\Feedback\Controllers\ModelConcerns;
use App\Modules\Core\Models\FeedbackFormFieldValue;
use App\Modules\Core\Models\FeedbackLog;
use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Report\Addon\Dashboards\Cards\TableCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Selfservice\Models\Article;
use App\Modules\Ticket\Models\Ticket;
use Form;

use function e;
use function get_class;
use function route;
use function trans;

/**
 * @template TModelClass of Ticket|Article
 * @template TFeedbackLog of FeedbackLog
 * @template TFeedbackFormFieldValue of FeedbackFormFieldValue
 * @extends TableCard<TModelClass>
 */
abstract class FeedbackLogTable extends TableCard
{
    /** @phpstan-use ModelConcerns<TModelClass, TFeedbackLog, TFeedbackFormFieldValue> */
    use ModelConcerns;

    /** @var string[] */
    protected array $with = ['feedback'];

    public function name(): string
    {
        return trans('report.feedback_log');
    }

    public function query(): QueryBuilder
    {
        /** @var class-string<TModelClass> $model */
        $model = get_class($this->model());
        $filters = $this->dashboard()->filters($this->getQualifiedName($this->logModel()->getTable(), 'updated_at'));

        return QueryBuilder::for($model)
            ->applyFilters($filters)
            ->join(
                $this->logModel()->getTable(),
                $this->logRelation()->getQualifiedParentKeyName(),
                '=',
                $this->logRelation()->getQualifiedForeignKeyName()
            )
            ->select($this->getQualifiedName($this->model()->getTable(), '*'))
            ->with('feedback')
            ->whereNotNull($this->getQualifiedName($this->logModel()->getTable(), 'rating'))
            ->orderBy($this->getQualifiedName($this->logModel()->getTable(), 'updated_at'), 'DESC');
    }

    protected function dateColumn(): Column
    {
        return (new Column(trans('general.date')))
            ->setId('updated_at')
            ->setValue(fn($model) => formatDate($model->feedback->updated_at));
    }

    protected function ratingColumn(): Column
    {
        return (new Column(trans('ticket.rating')))
            ->setValue(fn($model) => $model->feedback?->rating?->toString());
    }

    protected function actionsColumn(string $routeName): Column
    {
        return (new Column(''))
            ->setId('actions')
            ->setCsvValue(fn () => '')
            ->setValue(function ($model) use ($routeName) {
                $button = null;
                if ($model->feedback?->fields_answered) {
                    $url = e(route($routeName, [$model->feedback->id]));
                    $button = Form::button(
                        trans('ticket.view_feedback'),
                        [
                            'class' => 'view-feedback sp-button-sm sp-float-right sp-whitespace-nowrap',
                            'onclick' => "window.open('".$url."','width=800,height=600,scrollbars=yes');"
                        ]
                    )->toHtml();
                }

                return $button;
            }, false);
    }
}
