<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports\Cards\Responses;

use App\Http\Requests\Request;
use App\Modules\Core\Models\FeedbackFormFieldValue;
use App\Modules\Core\Models\FeedbackLog;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\ChartType;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\Datasets\PrimitiveDataset;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\DataStructure;
use App\Modules\Selfservice\Models\Article;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Database\Eloquent\Builder;
use UnhandledMatchError;

use function sprintf;
use function trans;

/**
 * @template TModel of Ticket|Article
 * @template TFeedbackLog of FeedbackLog
 * @template TFeedbackFormFieldValue of FeedbackFormFieldValue
 * @extends FieldChartCard<TModel, TFeedbackLog, TFeedbackFormFieldValue>
 */
abstract class BooleanFieldChart extends FieldChartCard
{
    public function data(): DataStructure
    {
        $results = $this->baseQuery()
            ->groupBy('value')
            ->selectRaw('count(*) as aggregate, value')
            ->getQuery()
            ->pluck('aggregate', 'value')
            ->toArray();

        return (new DataStructure(ChartType::PIE, new PrimitiveDataset([$results[1] ?? 0, $results[0] ?? 0])))
            ->setLabels([trans('general.yes'), trans('general.no')]);
    }

    /**
     * @inheritDoc
     */
    public function underlyingDataQuery(Request $request): Builder
    {
        $query = $this->baseQuery();
        if ($this->isDrillIntoChartRequest($request)) {
            $label = $request->string('label')->toString();

            if ($label === trans('general.yes')) {
                $query->where('value', 1);
            } elseif ($label === trans('general.no')) {
                $query->where('value', 0);
            } else {
                throw new UnhandledMatchError(sprintf('Unsupported label \'%s\'.', $label));
            }
        }

        return $query->with('feedback')
            ->selectRaw($this->getQualifiedName($this->model()->getTable(), '*'));
    }
}
