<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports\Cards\Responses;

use Addons\Reports\Feedback\Controllers\ModelConcerns;
use App\Http\Requests\Request;
use App\Modules\Core\Models\FeedbackFormField;
use App\Modules\Core\Models\FeedbackFormFieldValue;
use App\Modules\Core\Models\FeedbackLog;
use App\Modules\Report\Addon\Dashboards\Cards\ChartCard;
use App\Modules\Report\Addon\Dashboards\Cards\UnderlyingData;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Selfservice\Models\Article;
use App\Modules\Ticket\Models\Ticket;

use function get_class;
use function trans;

/**
 * @template TModel of Ticket|Article
 * @template TFeedbackLog of FeedbackLog
 * @template TFeedbackFormFieldValue of FeedbackFormFieldValue
 * @implements UnderlyingData<TModel>
 */
abstract class FieldChartCard extends ChartCard implements UnderlyingData
{
    /** @phpstan-use ModelConcerns<TModel, TFeedbackLog, TFeedbackFormFieldValue> */
    use ModelConcerns;

    protected FeedbackFormField $field;

    public function __construct(FeedbackFormField $field)
    {
        parent::__construct();

        $this->field = $field;
    }

    public function name(): string
    {
        return trans('report.feedback_field', ['name' => $this->field->name]);
    }

    /**
     * @return QueryBuilder<TModel>
     */
    protected function baseQuery(): QueryBuilder
    {
        /** @var class-string<TModel> $model */
        $model = get_class($this->model());
        $filters = $this->dashboard()->filters($this->getQualifiedName($this->logModel()->getTable(), 'updated_at'));

        return QueryBuilder::for($model)
            ->applyFilters($filters)
            ->join(
                $this->logModel()->getTable(),
                $this->logRelation()->getQualifiedParentKeyName(),
                '=',
                $this->logRelation()->getQualifiedForeignKeyName()
            )
            ->leftJoin(
                $this->valueModel()->getTable(),
                $this->getQualifiedName($this->logModel()->getTable(), 'id'),
                '=',
                $this->valueRelation()->getQualifiedForeignKeyName()
            )
            ->where(
                $this->getQualifiedName($this->valueModel()->getTable(), '.field_id'),
                $this->field->id
            );
    }

    protected function isDrillIntoChartRequest(Request $request): bool
    {
        return $request->has('label');
    }
}
