<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports\Tickets;

use Addons\Reports\Feedback\Reports\Cards\FeedbackLogTable as BaseFeedbackLogTable;
use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Ticket\Models\FeedbackFormFieldValue;
use App\Modules\Ticket\Models\FeedbackLog;
use App\Modules\Ticket\Models\Ticket;

use function array_merge;
use function e;
use function route;
use function sprintf;
use function trans_choice;

/**
 * @extends BaseFeedbackLogTable<Ticket, FeedbackLog, FeedbackFormFieldValue>
 */
class FeedbackLogTable extends BaseFeedbackLogTable
{
    public function __construct()
    {
        parent::__construct();

        $this->with = array_merge($this->with, ['user']);
    }

    public function columns(): array
    {
        return [
            $this->dateColumn(),
            $this->ticketColumn(),
            $this->userColumn(),
            $this->ratingColumn(),
            $this->actionsColumn('ticket.operator.feedback.show'),
        ];
    }

    public function model()
    {
        return new Ticket;
    }

    protected function ticketColumn(): Column
    {
        return (new Column(trans_choice('ticket.ticket', 1)))
            ->setCsvValue(fn(Ticket $model) => sprintf('#%s', $model->number))
            ->setValue(function (Ticket $model) {
                return sprintf('<a href="%s">#%s</a>', e($model->operatorUrl()), e($model->number));
            }, false);
    }

    protected function userColumn(): Column
    {
        return (new Column(trans_choice('user.user', 1)))
            ->setCsvValue(fn(Ticket $model) => $model->user?->formatted_name ?? '')
            ->setValue(function (Ticket $model) {
                return sprintf(
                    '<img src="%s" class="sp-avatar sp-max-w-2xs">&nbsp; <a href="%s">%s</a>',
                    e($model->user?->avatar_url ?? ''),
                    e(route('user.operator.user.edit', [$model->user_id])),
                    e($model->user?->formatted_name ?? '')
                );
            }, false);
    }
}
