<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports\Tickets\Responses;

use Addons\Reports\Feedback\Reports\Cards\Responses\FieldTable;
use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Ticket\Models\FeedbackFormFieldValue;
use App\Modules\Ticket\Models\FeedbackLog;
use App\Modules\Ticket\Models\Ticket;
use SupportPal\Purifier\Facades\Purifier;

use function e;
use function route;
use function trans;

/**
 * @extends FieldTable<Ticket, FeedbackLog, FeedbackFormFieldValue>
 */
class TextareaFieldTable extends FieldTable
{
    /** @var string[] */
    protected array $with = ['user'];

    public function columns(): array
    {
        return [$this->valueColumn()];
    }

    public function model()
    {
        return new Ticket;
    }

    protected function valueColumn(): Column
    {
        return (new Column(trans('report.feedback_responses')))
            ->setCsvValue(fn (Ticket $model) => $model->feedback?->values->firstOrFail()->value ?? '')
            ->setValue(function (Ticket $model) {
                $text = Purifier::clean($model->feedback?->values->firstOrFail()->value ?? '');
                $ticketroute = e($model->operatorUrl());
                $feedbackRoute = e(route('ticket.operator.feedback.show', [$model->feedback?->id]));
                $date = e(formatDate((int) $model->feedback?->updated_at));
                $ticketNumber = e($model->number);
                $userName = e($model->user?->formatted_name);
                $ratingColour = e($model->feedback?->rating?->textColour() ?? '');
                $ratingName = e($model->feedback?->rating?->toString() ?? '');
                $LANG = [
                    'ticket.rating' => e(trans('ticket.rating')),
                    'ticket.view_feedback' => e(trans('ticket.view_feedback')),
                ];

                return <<<EOT
${text}
<div class="sp-description sp-mt-3">
    <i class="fas fa-clock"></i>&nbsp;${date} &nbsp;
    <i class="fas fa-ticket"></i>&nbsp; <a target="_blank" href="${ticketroute}">#${ticketNumber}</a> &nbsp;
    ($userName) - ${LANG['ticket.rating']}: <span class="${ratingColour}">${ratingName}</span>
    - <a target="_blank" href="${feedbackRoute}">${LANG['ticket.view_feedback']}</a>
</div>
EOT;
            }, false);
    }
}
