<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports;

use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\DueTimeBreachesTable;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\FirstReplyTimeByPriorityCard;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\FirstReplyTimeCard;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\FirstReplyTimeTrendChart;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\NextReplyTimeByPriorityCard;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\NextReplyTimeCard;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\NextReplyTimeTrendChart;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\ResolutionTimeByPriorityCard;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\ResolutionTimeCard;
use Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance\ResolutionTimeTrendChart;
use App\Modules\Report\Addon\Dashboards\CardCollection;
use App\Modules\Report\Addon\Dashboards\Dashboard;
use App\Modules\Report\Addon\Dashboards\Filtering\Filter;
use App\Modules\Report\Addon\Dashboards\Filtering\Timeframe;
use App\Modules\Ticket\Models\TicketDueTimeTarget;

use function trans;

class SlaPerformance extends Dashboard
{
    public Timeframe $timeframe = Timeframe::ENABLE;
    public Filter $filter = Filter::TICKET;

    public function name(): string
    {
        return trans('report.sla_performance');
    }

    public function description(): string
    {
        return trans('report.sla_performance_desc');
    }

    public function loadCards(CardCollection $collection): void
    {
        if (TicketDueTimeTarget::where('target', TicketDueTimeTarget::TARGET_FIRST_REPLY_TIME)->exists()) {
            $collection->add(new FirstReplyTimeCard);
            $collection->add(new FirstReplyTimeTrendChart);
            $collection->add(new FirstReplyTimeByPriorityCard);
        }

        if (TicketDueTimeTarget::where('target', TicketDueTimeTarget::TARGET_NEXT_REPLY_TIME)->exists()) {
            $collection->add(new NextReplyTimeCard);
            $collection->add(new NextReplyTimeTrendChart);
            $collection->add(new NextReplyTimeByPriorityCard);
        }

        if (TicketDueTimeTarget::where('target', TicketDueTimeTarget::TARGET_RESOLVE_TIME)->exists()) {
            $collection->add(new ResolutionTimeCard);
            $collection->add(new ResolutionTimeTrendChart);
            $collection->add(new ResolutionTimeByPriorityCard);
        }

        $collection->add(new DueTimeBreachesTable);
    }
}
