<?php declare(strict_types=1);

namespace Addons\Reports\TicketVolume\Reports\Cards\Overview;

use App\Http\Requests\Request;
use App\Modules\Report\Addon\Dashboards\Cards\NumberCard;
use App\Modules\Report\Addon\Dashboards\Cards\UnderlyingData;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketCreatedAtColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketNumberColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketReopenedTimeColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketResolvedTimeColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketStatusColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketSubjectColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\UserFormattedNameColumn;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Database\Eloquent\Builder;

/**
 * @implements UnderlyingData<Ticket>
 */
abstract class TicketsNumberCard extends NumberCard implements UnderlyingData
{
    protected int $width = 4;
    protected int $height = 2;

    public function number(): ?float
    {
        return $this->baseQuery()->count();
    }

    /**
     * @inheritDoc
     */
    public function underlyingDataTableColumns(): array
    {
        return [
            new TicketCreatedAtColumn,
            new TicketNumberColumn,
            new TicketSubjectColumn,
            new UserFormattedNameColumn,
            new TicketStatusColumn,
            new TicketResolvedTimeColumn,
            new TicketReopenedTimeColumn,
        ];
    }

    /**
     * @inheritDoc
     */
    public function underlyingDataQuery(Request $request): Builder
    {
        return $this->baseQuery()
            ->with(['user', 'status'])
            ->selectRaw('ticket.*');
    }

    /**
     * @return Builder<Ticket>
     */
    abstract protected function baseQuery(): Builder;
}
