<?php declare(strict_types=1);

namespace Addons\Reports\TicketVolume\Reports\Cards\TicketsOpened;

use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketCreatedAtColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketDepartmentColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketNumberColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketStatusColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketSubjectColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketTagsColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\UserFormattedNameColumn;
use App\Modules\Ticket\Models\Tag;
use App\Modules\Ticket\Models\Ticket;

use function array_merge;
use function trans;

class ByTagTable extends BaseTable
{
    public function name(): string
    {
        return trans('report.tickets_opened_by_tag');
    }

    public function queryColumn(): string
    {
        return 'ticket_tag.id';
    }

    public function modelTable(): string
    {
        return Tag::getTableName();
    }

    /**
     * @inheritDoc
     */
    public function underlyingDataTableColumns(): array
    {
        return [
            new TicketCreatedAtColumn,
            new TicketNumberColumn,
            new TicketSubjectColumn,
            new UserFormattedNameColumn,
            new TicketDepartmentColumn,
            new TicketStatusColumn,
            new TicketTagsColumn
        ];
    }

    protected function underlyingDataRelations(): array
    {
        return array_merge(parent::underlyingDataRelations(), ['tags']);
    }

    /**
     * @inheritDoc
     */
    protected function baseQuery(): QueryBuilder
    {
        return QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters('ticket.created_at'))
            ->join('ticket_tag_membership', 'ticket.id', '=', 'ticket_tag_membership.ticket_id')
            ->join('ticket_tag', 'ticket_tag.id', '=', 'ticket_tag_membership.tag_id');
    }
}
