<?php declare(strict_types=1);

namespace Addons\Reports\TicketVolume\Reports\Cards\Workload;

use App\Modules\Report\Addon\Dashboards\Cards\Chart\ChartType;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\Datasets\PrimitiveDataset;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\DataStructure;
use App\Modules\Report\Addon\Dashboards\Cards\ChartCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Ticket\Models\Message;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Support\Facades\DB;

use function timeWithOffset;
use function trans;

class BusiestHourChart extends ChartCard
{
    protected int $width = 6;

    public function name(): string
    {
        return trans('report.busiest_hour_of_day');
    }

    public function data(): DataStructure
    {
        return (new DataStructure(ChartType::BAR, new PrimitiveDataset($this->getChartJsDataset())))
            ->setAxisNames(trans('report.hour_of_day'), trans('report.user_messages'))
            ->setOptions([
                'plugins' => [
                    'legend' => ['display' => false]
                ],
                'scales' => [
                    'y' => [
                        'ticks' => ['precision' => 0],
                    ],
                ],
            ]);
    }

    /**
     * @return mixed[]
     */
    private function getChartJsDataset(): array
    {
        $results = QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters('ticket_message.created_at'))
            ->leftJoin('ticket_message', 'ticket.id', '=', 'ticket_message.ticket_id')
            ->where('ticket_message.by', Message::USER)
            ->groupBy('hour')
            ->orderBy('hour')
            ->select([
                DB::raw('count(DISTINCT ticket_message.id) as aggregate'),
                // Need to apply timezone offset here to as we're using from_unixtime function
                DB::raw('hour(from_unixtime(`ticket_message`.`created_at` - ' . timeWithOffset(0) . ')) as hour')
            ])
            ->pluck('aggregate', 'hour');

        $data = [];
        for ($i = 1; $i <= 24; $i++) {
            $data[$i] = $results[$i] ?? 0;
        }

        return $data;
    }
}
