<?php declare(strict_types=1);

namespace Addons\Reports\TicketVolume\Reports\Cards\Workload;

use App\Modules\Report\Addon\Concerns\InteractsWithTimeframe;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\ChartType;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\Datasets\ObjectDataset;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\DataStructure;
use App\Modules\Report\Addon\Dashboards\Cards\ChartCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Ticket\Models\Message;
use App\Modules\Ticket\Models\Ticket;
use Carbon\CarbonInterface;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Collection;

use function get_object_vars;
use function sprintf;
use function trans;

class BusiestPeriodChart extends ChartCard
{
    use InteractsWithTimeframe;

    protected int $width = 12;

    public function name(): string
    {
        return trans('report.busiest_period');
    }

    public function data(): DataStructure
    {
        $unit = $this->calculateUnits();
        $dataset = new ObjectDataset($this->getChartJsDataset()->values()->toArray());

        return (new DataStructure(ChartType::LINE, $dataset))
            ->setAxisNames($this->unitToTrans($unit), trans('report.user_messages'))
            ->setOptions([
                'plugins' => [
                    'legend' => ['display' => false]
                ],
                'scales' => [
                    'x' => [
                        'type'    => 'time',
                        'time'    => [
                            'unit' => $unit,
                            'round' => $unit,
                            'tooltipFormat' => $this->unitToTooltipFormat($unit),
                        ],
                    ],
                    'y' => [
                        'min'   => 0,
                        'ticks' => ['precision' => 0],
                    ],
                ]
            ]);
    }

    /**
     * @return Builder<Ticket>
     */
    private function baseQuery(): Builder
    {
        return QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters($this->timeframeColumn()))
            ->leftJoin('ticket_message', 'ticket.id', '=', 'ticket_message.ticket_id')
            ->where('ticket_message.by', Message::USER);
    }

    /**
     * @return Collection<int,mixed>
     */
    private function getChartJsDataset(): Collection
    {
        $collection = $this->baseQuery()
            ->selectRaw(sprintf('%s * 1000 as x', $this->getTimeSeriesExpr($this->timeframeColumn())))
            ->selectRaw('COUNT(ticket_message.id) as y')
            ->groupBy('x')
            ->getQuery()
            ->get()
            ->keyBy('x');

        return $this->createDateRange($this->calculateUnits())
            ->map(function (CarbonInterface $d) use ($collection) {
                if (isset($collection[$d->unix() * 1000])) {
                    return get_object_vars($collection[$d->unix() * 1000]);
                }

                return ['x' => $d->unix() * 1000, 'y' => 0];
            });
    }

    private function timeframeColumn(): string
    {
        return 'ticket_message.created_at';
    }
}
