<?php declare(strict_types=1);

namespace Addons\Reports\UserStatistics\Reports\Cards\Demographics;

use App\Modules\Report\Addon\Dashboards\Cards\Number\NumberFormat;
use App\Modules\Report\Addon\Dashboards\Cards\NumberCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\User\Models\User;

use function trans;

class VerifiedUsers extends NumberCard
{
    protected NumberFormat $numberFormat = NumberFormat::PERCENT;

    public function name(): string
    {
        return trans('report.verified_users');
    }

    public function number(): ?float
    {
        $totalUsers = $this->totalUsers();
        $verifiedUsers = $this->verifiedUsers();

        return $totalUsers === 0 ? 0 : $verifiedUsers / $totalUsers;
    }

    private function totalUsers(): int
    {
        return $this->baseQuery()->count();
    }

    private function verifiedUsers(): int
    {
        return $this->baseQuery()
            ->where('user.email_verified', 1)
            ->count();
    }

    /**
     * @return QueryBuilder<User>
     */
    private function baseQuery(): QueryBuilder
    {
        return QueryBuilder::for(User::class)
            ->applyFilters($this->dashboard()->filters('user.created_at'))
            ->user();
    }
}
