<?php declare(strict_types=1);

namespace SupportPal\TopoClient\Locale;

use Illuminate\Support\Collection;
use SupportPal\TopoClient\Exceptions\FileNotFoundException;

use function file_get_contents;
use function json_decode;
use function sprintf;

/**
 * Dataset from https://github.com/lukes/ISO-3166-Countries-with-Regional-Codes/blob/master/all/all.json
 */
class Locales
{
    /** @var Collection<int, Locale> */
    private readonly Collection $locales;

    public function __construct()
    {
        $path = __DIR__.'/locales.json';
        $json = file_get_contents($path);
        if ($json === false) {
            throw new FileNotFoundException(sprintf('Unable to read %s.', $path));
        }

        $this->locales = new Collection;
        foreach (json_decode($json, true) as $locale) {
            $this->locales->push(new Locale($locale));
        }
    }

    public function findId(string $id): Locale
    {
        return $this->locales->where(fn (Locale $l) => $l->countryCode() === $id)->firstOrFail();
    }
}
