<?php declare(strict_types=1);

namespace SupportPal\HelpdeskRequirements\Database;

use Illuminate\Database\DatabaseManager;

class ClientVersion
{
    public const MIN_MYSQLND = '5.0.9';

    private DatabaseManager $db;
    private ?string $version = null;

    public function __construct(DatabaseManager $db)
    {
        $this->db = $db;
    }

    public function version(): string
    {
        if (isset($this->version)) {
            return $this->version;
        }

        return $this->version = $this->db->getClientVersion();
    }

    public function isMysqlNd(): bool
    {
        return stripos($this->version(), 'mysqlnd') !== false;
    }

    public function getMinimumSupported(): string
    {
        if ($this->isMysqlNd()) {
            return self::MIN_MYSQLND;
        }

        $serverVersion = new ServerVersion($this->db);

        return $serverVersion->getMinimumSupported();
    }

    public function isSupported(): bool
    {
        $version = $this->get();
        if ($version === null) {
            return false;
        }

        return version_compare($version, $this->getMinimumSupported(), '>=');
    }

    private function get(): ?string
    {
        $version = $this->version();
        preg_match('/((?:[0-9]{1,2}\.){1,3}[0-9]{1,2})/', $version, $matches);
        if (! isset($matches[1])) {
            return null;
        }

        return $matches[1];
    }
}
