<?php declare(strict_types=1);

namespace SupportPal\HelpdeskRequirements\Database;

use Illuminate\Database\DatabaseManager;
use SupportPal\Support\Database\Version\ServerVersion as Version;

class ServerVersion
{
    const MIN_MYSQL = '5.7';
    const MIN_MARIA = '10.2';

    private DatabaseManager $db;
    private ?Version $version = null;

    public function __construct(DatabaseManager $db)
    {
        $this->db = $db;
    }

    public function getMinimumSupported(): string
    {
        return $this->version()->isMaria() ? self::MIN_MARIA : self::MIN_MYSQL;
    }

    public function isSupported(): bool
    {
        return (bool) version_compare((string) $this->version(), $this->getMinimumSupported(), '>=');
    }

    public function version(): Version
    {
        if (isset($this->version)) {
            return $this->version;
        }

        return $this->version = $this->db->getServerVersion();
    }
}
