<?php declare(strict_types=1);

namespace SupportPal\HelpdeskRequirements;

use Illuminate\Database\DatabaseManager;
use Supportpal\HelpdeskBaseRequirements\Requirements\AbstractRequirement;
use SupportPal\HelpdeskRequirements\Database\ClientVersion;
use SupportPal\HelpdeskRequirements\Database\ServerVersion;
use SupportPal\Requirements\Requirement;
use SupportPal\Requirements\RequirementGroup;
use SupportPal\Requirements\Translation\Translation;

class DatabaseRequirements extends AbstractRequirement
{
    private DatabaseManager $db;

    public function __construct(DatabaseManager $db)
    {
        $this->db = $db;
    }

    public function getRequirementGroup(): RequirementGroup
    {
        $serverVersion = new ServerVersion($this->db);
        $clientVersion = new ClientVersion($this->db);

        return new RequirementGroup(
            new Translation('installer.mysql_version'),
            [
                new Requirement(
                    fn() => $serverVersion->isSupported(),
                    new Translation('installer.mysql_server_version', ['min' => $serverVersion->getMinimumSupported()]),
                    new Translation('installer.mysql_version_not_found', ['version' => (string) $serverVersion->version()])
                ),
                new Requirement(
                    fn() => $clientVersion->isSupported(),
                    new Translation('installer.mysql_client_version', ['min' => $clientVersion->getMinimumSupported()]),
                    new Translation('installer.mysql_version_not_found', ['version' => $clientVersion->version()])
                ),
            ],
            [],
            null,
            new Translation('installer.help_mysql_version')
        );
    }
}
