<?php declare(strict_types=1);

namespace SupportPal\Support\Database\Version;

use PDO;

use function preg_replace;
use function stripos;

class ServerVersion
{
    private string $version;

    public function __construct(PDO $db)
    {
        $this->version = (string) $db->getAttribute(PDO::ATTR_SERVER_VERSION);
    }

    public function __toString(): string
    {
        // MariaDB: Strip off any leading '5.5.5-', if present
        // https://jira.mariadb.org/browse/MDEV-4575
        if ($this->isMaria()) {
            return (string) preg_replace('/^5\.5\.5-/', '', $this->version);
        }

        return $this->version;
    }

    public function isMaria(): bool
    {
        return stripos($this->version, 'mariadb') !== false;
    }
}
