<?php declare(strict_types=1);

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('user_export', function (Blueprint $table) {
            $table->engine = 'InnoDB';

            $table->charset = 'utf8mb4';
            $table->collation = 'utf8mb4_unicode_ci';

            $table->increments('id')->unsigned();

            $table->integer('export_id')->unsigned();
            $table->foreign('export_id')->references('id')->on('export')->onDelete('cascade');

            $table->integer('user_id')->unsigned();
            $table->foreign('user_id')->references('id')->on('user')->onDelete('cascade');

            $table->integer('created_at');
            $table->integer('updated_at')->default(0);
        });

        $exports = DB::table('export')->whereNotNull('user_id')->get();

        foreach ($exports as $export) {
            DB::table('user_export')->insert([
                'export_id'  => $export->id,
                'user_id'    => $export->user_id,
                'created_at' => $export->created_at,
                'updated_at' => $export->updated_at,
            ]);
        }
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('user_export');
    }
};
