<?php declare(strict_types=1);

namespace Database\Seeds\v5_0_0;

use App\Modules\Core\Controllers\Cache\WarmupCacheTask;
use App\Modules\Core\Controllers\Database\Seed\Seeder;
use App\Modules\Core\Controllers\Queues\DefaultQueueWorker;
use App\Modules\Core\Controllers\Queues\SlowQueueWorker;
use App\Modules\Core\Controllers\Search\ScoutIndexTask;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;

class SeedScheduledTaskTable extends Seeder
{
    /**
     * Run the database seeds.
     *
     * @return void
     */
    public function run()
    {
        DB::table('scheduled_task')->insert([
            [
                'name'        => 'Index scout models',
                'description' => 'Indexes model data for search.',
                'class'       => ScoutIndexTask::class,
                'interval'    => 45,
                'last_run'    => 0,
                'created_at'  => Carbon::now()->getTimestamp(),
                'updated_at'  => Carbon::now()->getTimestamp(),
            ],
            [
                'name'        => 'Warm up cache',
                'description' => 'Populate the cache to improve system performance.',
                'class'       => WarmupCacheTask::class,
                'interval'    => 285,
                'last_run'    => 0,
                'created_at'  => Carbon::now()->getTimestamp(),
                'updated_at'  => Carbon::now()->getTimestamp(),
            ],
            [
                'name'        => 'Queue worker (default)',
                'description' => 'Process jobs on the default queue.',
                'class'       => DefaultQueueWorker::class,
                'interval'    => 45,
                'last_run'    => 0,
                'created_at'  => Carbon::now()->getTimestamp(),
                'updated_at'  => Carbon::now()->getTimestamp(),
            ],
            [
                'name'        => 'Queue worker (slow)',
                'description' => 'Process jobs on the slow queue.',
                'class'       => SlowQueueWorker::class,
                'interval'    => 45,
                'last_run'    => 0,
                'created_at'  => Carbon::now()->getTimestamp(),
                'updated_at'  => Carbon::now()->getTimestamp(),
            ]
        ]);

        DB::table('scheduled_task')
            ->where('class', 'LIKE', 'App\\\\Modules\\\\Core\\\\Models\\\\ExportQueue')
            ->delete();
    }
}
