<?php declare(strict_types=1);

namespace Database\Seeds\v5_0_0;

use App\Modules\Core\Controllers\Database\Seed\Seeder;
use App\Modules\Core\Models\EmailTemplate;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

use function file_exists;
use function file_get_contents;
use function implode;
use function now;
use function sprintf;

class SeedEmailTemplateDataTable extends Seeder
{
    public function run(): void
    {
        $ids = [6, 9, 12, 22, 26, 32];

        DB::update(sprintf("UPDATE `email_template_data`
            SET `contents` = REPLACE(`contents`, '{{ confirmation_link }}', '{{ verification_link }}'),
                `contents` = REPLACE(`contents`, ' confirm ', ' verify '),
                `contents` = REPLACE(`contents`, ' confirmed,', ' verified,')
            WHERE `template_id` IN (%s)", implode(',', $ids)));

        $this->updateTemplate(6, 'Password reset', 'Password reset verification');
        $this->updateTemplate(9, 'Password reset', 'Password reset verification');
        $this->updateTemplate(12, 'Organisation invitation', 'Invited to join organisation');
        $this->updateTemplate(22, 'Account verification', 'Account verification');
        $this->updateTemplate(30, 'Export ready', 'Export ready', __DIR__ . '/../2.3.1/emails/template_30.twig');
        $this->updateTemplate(32, 'Verify new email address', 'Verify your new email address');

        $time = now()->getTimestamp();

        if (DB::table('email_template')
            ->where('id', EmailTemplate::USER_ADDITIONAL_EMAIL_ADDED_NOTIFICATION)
            ->where('type', EmailTemplate::USER)
            ->count()
        ) {
            DB::table('email_template_data')->insert([
                [
                    'template_id' => EmailTemplate::USER_ADDITIONAL_EMAIL_ADDED_NOTIFICATION,
                    'subject'     => 'An email address was added to your account',
                    'contents'    => file_get_contents(__DIR__ . '/emails/template_33.twig'),
                    'created_at'  => $time,
                    'updated_at'  => $time
                ]
            ]);
        } else {
            Log::error(sprintf('Unable to insert email template %s, already exists in database.', EmailTemplate::USER_ADDITIONAL_EMAIL_ADDED_NOTIFICATION));
        }

        if (DB::table('email_template')
            ->where('id', EmailTemplate::USER_VERIFY_EMAIL_NOTIFICATION)
            ->where('type', EmailTemplate::USER)
            ->count()
        ) {
            DB::table('email_template_data')->insert([
                [
                    'template_id' => EmailTemplate::USER_VERIFY_EMAIL_NOTIFICATION,
                    'subject'     => 'Please verify your email address',
                    'contents'    => file_get_contents(__DIR__ . '/emails/template_34.twig'),
                    'created_at'  => $time,
                    'updated_at'  => $time
                ]
            ]);
        } else {
            Log::error(sprintf('Unable to insert email template %s, already exists in database.', EmailTemplate::USER_VERIFY_EMAIL_NOTIFICATION));
        }
    }

    /**
     * Overwrite email template if the updated time is the same as the created time.
     */
    private function updateTemplate(int $id, string $name, string $subject, ?string $path = null): void
    {
        // Check if the template has been modified.
        $unmodified = DB::table('email_template_data')
            ->where('template_id', $id)
            ->whereColumn('created_at', 'updated_at')
            ->whereNull('brand_id')
            ->whereNull('language_code')
            ->count();

        if (! $unmodified) {
            return;
        }

        DB::table('email_template')
            ->where('id', $id)
            ->update(['name' => $name]);

        $data = ['subject' => $subject];

        if (! isset($path)) {
            $path = __DIR__ . '/../2.0.0-beta1/emails/template_' . $id . '.twig';
        }

        if (file_exists($path)) {
            $data['contents'] = file_get_contents($path);
        }

        DB::table('email_template_data')
            ->where('template_id', $id)
            ->whereColumn('created_at', 'updated_at')
            ->whereNull('brand_id')
            ->whereNull('language_code')
            ->update($data);
    }
}
