<?php declare(strict_types=1);

namespace Database\Seeds\v5_0_3;

use App\Modules\Core\Controllers\Database\Seed\Seeder;
use App\Modules\Report\Jobs\SeedFirstResponseTimeMetric;
use App\Modules\Report\Jobs\SeedMetrics;
use App\Modules\Report\Metrics\Metric;
use Illuminate\Support\Facades\DB;

use function collect;
use function now;

class SeedReportMetricValueTable extends Seeder
{
    public function run(): void
    {
        $time = now()->timestamp;

        $metrics = collect([
            Metric::FIRST_RESPONSE_TIME,
            Metric::FIRST_OPERATOR_REPLY,
        ]);

        DB::table('report_metric')->whereIn('name', $metrics)->delete();

        $metrics = $metrics->map(function (Metric $metric) use ($time) {
            return [
                'name'       => $metric->value,
                'created_at' => $time,
                'updated_at' => $time
            ];
        })
            ->toArray();

        DB::table('report_metric')->insert($metrics);

        SeedMetrics::dispatch([SeedFirstResponseTimeMetric::class]);
    }
}
