<?php declare(strict_types=1);

namespace SupportPal\BinPacking;

use ArrayIterator;
use Countable;
use IteratorAggregate;
use Traversable;

use function array_reduce;
use function count;

class Bin implements IteratorAggregate, Countable
{
    /** @var Item[] */
    protected array $items = [];

    protected int $maximumBinWeight;

    public function __construct(int $maximumBinWeight)
    {
        $this->maximumBinWeight = $maximumBinWeight;
    }

    /** @return Item[] */
    public function getItems(): array
    {
        return $this->items;
    }

    public function addItem(int|string $key, Item $item): Bin
    {
        $this->items[$key] = $item;

        return $this;
    }

    public function getTotalWeight() : int
    {
        return array_reduce($this->items, function ($acc, $item) {
            return $acc + $item->getWeight();
        }, 0);
    }

    public function itemFits(Item $item): bool
    {
        return $item->getWeight() + $this->getTotalWeight() <= $this->maximumBinWeight;
    }

    public function getIterator(): Traversable
    {
        return new ArrayIterator($this->items);
    }

    public function count(): int
    {
        return count($this->items);
    }
}
