<?php declare(strict_types=1);

namespace Chumper\Datatable\Columns;

use function is_array;
use function is_string;

class DateColumn extends BaseColumn
{
    /**
     * Constants for the time representation
     */
    const DATE = 0;
    const TIME = 1;
    const DATE_TIME = 2;
    const CUSTOM = 4;
    const FORMATTED_DATE = 5;
    const DAY_DATE = 6;

    private int $format;
    private string $custom;

    public function __construct(string $name, int $format = 2, string $custom = '')
    {
        parent::__construct($name);
        $this->format = $format;
        $this->custom = $custom;
    }

    /**
     * @param mixed $model The data to pass to the column,
     *              could be a model or an array
     * @return mixed the return value of the implementation,
     *              should be text in most of the cases
     */
    public function run(mixed $model): mixed
    {
        if (is_string(is_array($model) ? $model[$this->name] : $model->{$this->name})) {
            return is_array($model) ? $model[$this->name] : $model->{$this->name};
        }

        return match ($this->format) {
            DateColumn::DATE => is_array($model) ? $model[$this->name]->toDateString() : $model->{$this->name}->toDateString(),
            DateColumn::TIME => is_array($model) ? $model[$this->name]->toTimeString() : $model->{$this->name}->toTimeString(),
            DateColumn::DATE_TIME => is_array($model) ? $model[$this->name]->toDateTimeString() : $model->{$this->name}->toDateTimeString(),
            DateColumn::CUSTOM => is_array($model) ? $model[$this->name]->format($this->custom) : $model->{$this->name}->format($this->custom),
            DateColumn::FORMATTED_DATE => is_array($model) ? $model[$this->name]->toFormattedDateString() : $model->{$this->name}->toFormattedDateString(),
            DateColumn::DAY_DATE => is_array($model) ? $model[$this->name]->toDayDateTimeString() : $model->{$this->name}->toDayDateTimeString(),
        };
    }
}
