<?php declare(strict_types=1);

namespace Chumper\Datatable;

use Illuminate\Contracts\Support\DeferrableProvider;
use Illuminate\Support\ServiceProvider;

use function base_path;
use function config_path;

class DatatableServiceProvider extends ServiceProvider implements DeferrableProvider
{
    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        $this->publishes([
            __DIR__.'/../../config/config.php' => config_path('chumper.datatable.php'),
            __DIR__.'/../../views' => base_path('resources/views/vendor/chumper.datatable'),
        ]);

        $this->loadViewsFrom(__DIR__ . '/../../views', 'chumper.datatable');
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(__DIR__.'/../../config/config.php', 'chumper.datatable');

        $this->app->singleton('datatable', function ($app) {
            $datatable = new Datatable($app['request']);
            $app->refresh('request', $datatable, 'setRequest');

            return $datatable;
        });
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['datatable'];
    }
}
