<?php declare(strict_types=1);

namespace Chumper\Datatable\Engines\Dataset;

use Chumper\Datatable\Engines\BaseEngine;
use Illuminate\Support\Collection;
use Illuminate\Support\LazyCollection;

class CollectionDataset extends Dataset
{
    private int $skip = 0;
    private ?int $take = null;

    public function __construct(private Collection $collection, BaseEngine $engine)
    {
        parent::__construct($engine);

        $this->collection = $collection->map(fn($row) => $this->makeEntry($row));
    }

    public function set(Collection $collection): self
    {
        $this->collection = $collection;

        return $this;
    }

    public function setPagination(int $skip, ?int $take): self
    {
        $this->skip = $skip;
        $this->take = $take;

        return $this;
    }

    public function raw(): Collection
    {
        return $this->collection;
    }

    public function get(): Collection
    {
        return $this->collection
            ->slice($this->skip, $this->take)
            ->values();
    }

    public function lazy(): LazyCollection
    {
        return $this->get()->lazy();
    }
}
