<?php declare(strict_types=1);

namespace Chumper\Datatable\Engines\Dataset;

use Chumper\Datatable\Engines\BaseEngine;
use Illuminate\Support\Collection;
use Illuminate\Support\LazyCollection;

use function call_user_func;
use function is_callable;

abstract class Dataset
{
    public function __construct(private readonly BaseEngine $engine)
    {
        //
    }

    abstract public function get(): Collection;

    abstract public function lazy(): LazyCollection;

    protected function makeEntry(mixed $row): array
    {
        $entry = [];
        if (is_callable($this->engine->getRowClass())) {
            $entry['DT_RowClass'] = call_user_func($this->engine->getRowClass(), $row);
        }

        if (is_callable($this->engine->getRowId())) {
            $entry['DT_RowId'] = call_user_func($this->engine->getRowId(), $row);
        }

        if (is_callable($this->engine->getRowData())) {
            $entry['DT_RowData'] = call_user_func($this->engine->getRowData(), $row);
        }

        $i = 0;
        foreach ($this->engine->getColumns() as $col) {
            if ($this->engine->getAliasMapping()) {
                $entry[$col->getName()] = $col->run($row);
            } else {
                $entry[$i] = $col->run($row);
            }

            $i++;
        }

        return $entry;
    }
}
