<?php declare(strict_types=1);

namespace SupportPal\HealthService;

use Illuminate\Config\Repository;
use Illuminate\Contracts\Cache\Repository as CacheRepository;
use Illuminate\Contracts\Support\DeferrableProvider;
use Illuminate\Support\ServiceProvider;
use SupportPal\HealthService\Check\HealthCheck;
use SupportPal\HealthService\Service\HealthChecker;
use Traversable;

use function iterator_to_array;

class HealthServiceProvider extends ServiceProvider implements DeferrableProvider
{
    public function register(): void
    {
        $this->app->bind(HealthChecker::class, function () {
            /** @var Traversable<HealthCheck> $iterator */
            $iterator = $this->app->tagged(HealthCheck::class);

            return new HealthChecker(
                iterator_to_array($iterator),
                $this->app->make(CacheRepository::class),
                $this->app->make(Repository::class)
            );
        });
    }

    /**
     * @return class-string[]
     */
    public function provides()
    {
        return [HealthChecker::class];
    }
}
