<?php declare(strict_types=1);

namespace SupportPal\HealthService\Model;

use Illuminate\Support\Carbon;

class Result
{
    public const OK = 'OK';
    public const WARNING = 'warning';
    public const ERROR = 'ERR';
    public const SKIP = 'SKIP';

    /** @var bool */
    private $healthy;
    /** @var string */
    private $state;
    /** @var ResultString|string */
    private $name;
    /** @var ResultString|string */
    private $message;
    /** @var int */
    private $timestamp;

    /**
     * @param ResultString|string $name
     * @param ResultString|string $message
     */
    public static function skip($name, $message): Result
    {
        return new self(true, self::SKIP, $name, $message);
    }

    /**
     * @param ResultString|string $name
     * @param ResultString|string $message
     */
    public static function healthy($name, $message): Result
    {
        return new self(true, self::OK, $name, $message);
    }

    /**
     * @param ResultString|string $name
     * @param ResultString|string $message
     */
    public static function warning($name, $message, bool $healthy = true): Result
    {
        return new self($healthy, self::WARNING, $name, $message);
    }

    /**
     * @param ResultString|string $name
     * @param ResultString|string $message
     */
    public static function error($name, $message): Result
    {
        return new self(false, self::ERROR, $name, $message);
    }

    /**
     * @param ResultString|string $name
     * @param ResultString|string $message
     */
    private function __construct(bool $healthy, string $state, $name, $message)
    {
        $this->healthy = $healthy;
        $this->state = $state;
        $this->name = $name;
        $this->message = $message;
        $this->timestamp = Carbon::now()->getTimestamp();
    }

    public function isHealthy(): bool
    {
        return $this->healthy;
    }

    public function getState(): string
    {
        return $this->state;
    }

    public function getName(): string
    {
        if ($this->name instanceof ResultString) {
            return $this->name->get();
        }

        return $this->name;
    }

    public function getMessage(): string
    {
        if ($this->message instanceof ResultString) {
            return $this->message->get();
        }

        return $this->message;
    }

    public function getLastRun(): int
    {
        return $this->timestamp;
    }
}
