<?php declare(strict_types=1);

namespace SupportPal\HelpdeskRequirements;

use Illuminate\Contracts\Config\Repository;
use Supportpal\HelpdeskBaseRequirements\PlatformRequirements;
use Supportpal\HelpdeskBaseRequirements\Requirements\AbstractRequirement;
use Supportpal\HelpdeskBaseRequirements\Requirements\FilePermissions;
use SupportPal\Requirements\RequirementGroup;
use SupportPal\Requirements\Requirements as RequirementsList;

class FilePermissionRequirements extends AbstractRequirement
{
    private Repository $config;

    public function __construct(Repository $config)
    {
        $this->config = $config;
    }

    public function getRequirementGroup(): RequirementGroup
    {
        $basePath = base_path();

        $appPath = $this->config->get('filesystems.disks.local.root', storage_path() . '/app');

        $directories = [
            $basePath . '/config',
            $appPath,
            $appPath . '/attachments',
            $appPath . '/email_cache',
            $appPath . '/email_log',
            $appPath . '/exports',
            $appPath . '/public',
            $appPath . '/purifier_cache',
            $appPath . '/selfservice',
            $appPath . '/tickets',
            $this->config->get('cache.stores.file.path', storage_path() . '/framework/cache'),
            $this->config->get('cache.stores.file.lock_path', storage_path() . '/framework/locks'),
            $this->config->get('session.files', storage_path() . '/framework/sessions'),
            $this->config->get('view.compiled', storage_path() . '/framework/views'),
            $this->config->get('app.log_path', storage_path() . '/logs'),
        ];

        $group = RequirementsList::getInstance()->get(PlatformRequirements::FILE_PERMISSIONS);
        foreach ($directories as $directory) {
            $group->addRequiredRequirement(
                (new FilePermissions($basePath))->makeDirPermissionRequirement($directory)
            );
        }

        return $group;
    }
}
