<?php declare(strict_types=1);

namespace SupportPal\Marketplace\Http;

use GuzzleHttp\MessageFormatterInterface;
use GuzzleHttp\Psr7\Message;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Throwable;

use function rewind;
use function sprintf;
use function stream_get_contents;

class MessageFormatter implements MessageFormatterInterface
{
    /** @var resource|null */
    private $stream;

    /**
     * @param resource|null $stream
     */
    public function __construct($stream)
    {
        $this->stream = $stream;
    }

    public function format(RequestInterface $request, ?ResponseInterface $response = null, ?Throwable $error = null): string
    {
        return sprintf(
            ">>>>>>>>\n%s\n%s\n<<<<<<<<\n%s\n--------\n%s",
            Message::toString($request),
            $this->getStreamContent(),
            $response ? Message::toString($response) : '',
            $error ? $error->getMessage() : 'NULL'
        );
    }

    private function getStreamContent(): string
    {
        if ($this->stream === null) {
            return '';
        }

        rewind($this->stream);

        return stream_get_contents($this->stream) ?: '';
    }
}
