<?php declare(strict_types=1);

namespace SupportPal\Purifier;

use HTMLPurifier;
use HTMLPurifier_Config;
use SupportPal\DomUtils\Html\Filter\MakeWellFormed;
use SupportPal\DomUtils\Html\Html;
use SupportPal\Purifier\Config\Config;
use SupportPal\Purifier\Twig\Replacer;

class Purifier
{
    protected Config $config;
    protected HTMLPurifier $purifier;

    public function __construct(Config $config)
    {
        $this->config = $config;
        $this->purifier = new HTMLPurifier($this->config);
    }

    /**
     * Remove malicious HTML.
     *
     * @param array<mixed> $config
     */
    public function clean(string $dirty, ?array $config = null): string
    {
        $dirty = (new Html($dirty))->filter([MakeWellFormed::class])->getHtml();

        return $this->getInstance()->purify($dirty, $this->config($config));
    }

    /**
     * Run HTMLPurifier over a twig template.
     *
     * @param  string  $dirty
     * @param  array<string>   $config
     * @return string
     */
    public function cleanTwig($dirty, ?array $config = null)
    {
        $replacer = new Replacer($dirty);

        return $replacer->run(function ($string) use ($config) {
            return $this->clean($string, $config);
        });
    }

    /**
     * Get HTMLPurifier instance.
     *
     * @return HTMLPurifier
     */
    public function getInstance()
    {
        return $this->purifier;
    }

    /**
     * If you pass a custom configuration to the HTMLPurifier purify function
     * it replaces the preconfigured config instance rather than merges, see
     * comment:
     *  > // make the config merge in, instead of replace
     *
     * If a custom configuration is provided, this ensures that it's merged in
     * with the defaults.
     *
     * @param array<mixed>|null $config
     * @return HTMLPurifier_Config
     */
    public function config(?array $config = null): HTMLPurifier_Config
    {
        return $this->config->make($config);
    }
}
